/*
    Copyright (C) 1999-2001 Rui Miguel Silva Seabra <rms@greymalkin.yi.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#include "irmctl_readconf.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <glib.h>

pTELECOMMAND parseTeleCommand(xmlDocPtr tc_xml) {
	pTELECOMMAND tc = (pTELECOMMAND)g_malloc(sizeof(TELECOMMAND));
	xmlNodePtr root = xmlDocGetRootElement(tc_xml);
	xmlNodePtr child = root->childs;
	gchar *model, *brand, *type, *mode, *code, *name;
	gpointer oldvalue;

	tc->model = g_strdup(model = (gchar*)xmlGetProp(root, (xmlChar*)"MODEL"));
	tc->brand = g_strdup(brand = (gchar*)xmlGetProp(root, (xmlChar*)"BRAND"));
	tc->type = g_strdup(type = (gchar*)xmlGetProp(root, (xmlChar*)"TYPE"));
	tc->mode = g_strdup(mode = (gchar*)xmlGetProp(root, (xmlChar*)"MODE"));
#ifdef DEBUG
	printf("\t\t%s; %s; %s; %s\n", model, brand, type, mode);
#endif
	tc->nameof=g_hash_table_new(g_str_hash, g_str_equal);
	while(child) {
		if(!g_strcasecmp((gchar*)child->name, "BUTTON")) {
			oldvalue=NULL;
			code = g_strdup((gchar*)xmlGetProp(child, (xmlChar*)"CODE"));
			name = g_strdup((gchar*)xmlGetProp(child, (xmlChar*)"NAME"));
#ifdef DEBUG
	printf("\t\tTrying to set value of %s\n", code);
#endif
			if((oldvalue = g_hash_table_lookup(tc->nameof, code))) {
#ifdef DEBUG
	printf("\t\t\tFreeing old value of %s\n", code);
#endif
				g_free(oldvalue);
			}
#ifdef DEBUG
	printf("\t\t\tSet %s = %s\n", code, name);
#endif
			g_hash_table_insert(tc->nameof, code, name);

		}
		child=child->next;
	}

	return tc;
}
pIRMCTL irmctl_loadTeleCommand(xmlNodePtr node, pIRMCTL conf) {
	gchar * path = (gchar*)xmlGetProp(node, (xmlChar*)"SRC");
	xmlDocPtr tc_xml;

	tc_xml = xmlParseFile(path);
	if (tc_xml) {
		conf->tc = parseTeleCommand(tc_xml);
		xmlFreeDoc(tc_xml);

	} else {
		g_warning("Could not open %s: %s\n", path, g_strerror(errno));
		exit(EXIT_FAILURE);
	}
	return conf;
}

pIRMCTL irmctl_loadSwitchButton(xmlNodePtr node, pIRMCTL conf) {
	gchar * sb = (gchar*)xmlGetProp(node, (xmlChar*)"NAME");

	// FIXME: Have to find a good way to check the validity.
	//        It is bad to define a switch without the button
	conf->switchbutton=g_strdup(sb);
	return conf;
}

pIRMCTL irmctl_loadTenButton(xmlNodePtr node, pIRMCTL conf) {
	gchar * tb = (gchar*)xmlGetProp(node, (xmlChar*)"NAME");

	// FIXME: Have to find a good way to check the validity.
	//        It is bad to define a switch without the button
	conf->tenbutton=g_strdup(tb);
	return conf;
}

pIRMCTL irmctl_loadGlobals(xmlNodePtr node, pIRMCTL conf) {
	xmlNodePtr child=node->childs;
	while(child) {
#ifdef DEBUG
		printf("\t<%s>\n", child->name);
#endif
		if(!g_strcasecmp((gchar*)child->name,"TELECOMMAND")) {
			conf = irmctl_loadTeleCommand(child, conf);
		} else if (!g_strcasecmp((gchar*)child->name,"SWITCHBUTTON")) {
			conf = irmctl_loadSwitchButton(child, conf);
		} else if (!g_strcasecmp((gchar*)child->name,"TENBUTTON")) {
			conf = irmctl_loadTenButton(child, conf);
		}
		child=child->next;
	}
	return conf;
}

#ifdef __IRMCTL_NO_BUILTINS
pIRMCTL irmctl_loadBuiltIns(xmlNodePtr node, pIRMCTL conf) {
	return conf;
}
#else

#ifdef __IRMCTL_BUILTIN_XMMS
#include "irmctl_readconf_xmms.c"
#endif

pIRMCTL irmctl_loadBuiltIns(xmlNodePtr node, pIRMCTL conf) {
	xmlNodePtr builtin=node->childs;

#ifdef DEBUG
		printf("\t<%s>\n", builtin->name);
#endif
	while(builtin) {
#ifdef __IRMCTL_BUILTIN_XMMS
		if(!g_strcasecmp((gchar*)builtin->name,"XMMS")) {
			conf = irmctl_loadBuiltInXMMS(builtin, conf);
		}
#endif
		builtin=builtin->next;
	}
	return conf;
}
#endif

pIRMCTL irmctl_loadMode(xmlNodePtr mode, pIRMCTL conf) {
	xmlNodePtr button = mode->childs;
	pMODE Mode=NULL;
	gchar *modeName = (gchar*)xmlGetProp(mode, (xmlChar*)"NAME");
	gchar *switchBut = (gchar*)xmlGetProp(mode, (xmlChar*)"SWITCH");
	gchar *butName, *command, *aux;
	pACTION a;


	if(!g_hash_table_lookup(conf->modes, modeName)) {
		Mode = (pMODE)g_malloc(sizeof(MODE));
		Mode->actions = g_hash_table_new(g_str_hash, g_str_equal);
		Mode->name = g_strdup(modeName);
		while(button) {
			a=(pACTION)g_malloc(sizeof(ACTION));
			//gchar**zbr=NULL;
			butName = (gchar*)xmlGetProp(button, (xmlChar*)"NAME");
			command=(gchar*)xmlNodeGetContent(button->childs);
			a->type=0;
			a->handler.command = g_strsplit(command, " ", 10);
			if((aux=g_hash_table_lookup(Mode->actions, butName)))
				g_strfreev((gchar**)aux);
			g_hash_table_insert(Mode->actions, g_strdup(butName),a);
			//zbr=g_hash_table_lookup(Mode->commands, butName);
			//g_message("Command loaded: %s\n", zbr[0]);
	
			button=button->next;
		}
		g_hash_table_insert(conf->modes, g_strdup(switchBut), Mode);
	}
	if(!conf->currentMode && Mode) conf->currentMode=Mode;
	return conf;
}

pIRMCTL irmctl_loadModes(xmlNodePtr node, pIRMCTL conf) {
	xmlNodePtr mode=node->childs;

	while(mode) {
#ifdef DEBUG
		printf("\t<%s>\n", (char*)xmlGetProp(mode, (xmlChar*)"NAME"));
#endif
		conf = irmctl_loadMode(mode, conf);
		mode=mode->next;
	}
	return conf;
}



pIRMCTL parseConf(xmlDocPtr conf_xml, gchar *path) {
	xmlNodePtr child;
	pIRMCTL conf = (pIRMCTL)g_malloc(sizeof(IRMCTL));
	
	if (!conf) irmctl_error_memory("Failed to load configuration");
	conf->doc=path;
	conf->modes=NULL;
	conf->currentMode=NULL;
	conf->modes=g_hash_table_new(g_str_hash, g_str_equal);
	child = (xmlDocGetRootElement(conf_xml))->childs;
	while(child) {
#ifdef DEBUG
		printf("<%s>\n", child->name);
#endif
		if(!g_strcasecmp((gchar*)child->name,"GLOBALS")) {
			conf = irmctl_loadGlobals(child, conf);
		} else if (!g_strcasecmp((gchar*)child->name,"BUILTINS")) {
			conf = irmctl_loadBuiltIns(child, conf);
		} else if (!g_strcasecmp((gchar*)child->name,"MODES")) {
			conf = irmctl_loadModes(child, conf);
		}
		child=child->next;
	}
	return conf;
}

pIRMCTL irmctl_loadconf(gchar *path) {
	xmlDocPtr conf_xml;
	pIRMCTL conf;

	conf_xml = xmlParseFile(path);
	if (conf_xml) {
		conf = parseConf(conf_xml, path);
		 xmlFreeDoc(conf_xml);

	} else {
		g_warning("Could not open %s: %s\n", path, g_strerror(errno));
		conf = NULL;
	}
	return conf;
}
