/*
    Copyright (C) 1999-2001 Rui Miguel Silva Seabra <rms@greymalkin.yi.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef _IRMCTL_CONF
 #define _IRMCTL_CONF

#include <stdlib.h>
#include <string.h>

#include <parser.h>
#include <tree.h>

#include <unistd.h>
#include <syslog.h>
#include <glib.h>
#include <xmms/xmmsctrl.h>

#define TCSIZE 12

typedef struct TeleCommand {
	gchar *model;
	gchar *brand;
	gchar *type;
	gchar *mode;
	GHashTable* nameof;
} *pTELECOMMAND, TELECOMMAND;

typedef struct Mode {
	gchar *name;
	GHashTable *actions;
} *pMODE, MODE;

typedef struct Function {
	gint type;
	void (*args)();
	gint a1type;
	union arg1 {
		gint intarg;
		gchar *chararg;
	} arg1;
	gint a2type;
	union arg2 {
		gint intarg;
		gchar *chararg;
	} arg2;
} *pFUNCTION, FUNCTION;

typedef struct Action {
	gshort type;
	union handler {
		pFUNCTION func;
		gchar **command;
	} handler;
} *pACTION, ACTION;

typedef struct Conf {
	pTELECOMMAND tc;
	gchar *switchbutton;
	gchar *tenbutton;
	gchar *telecpath;
	gchar *confpath;
	gchar *doc;
	GHashTable * modes;
	pMODE currentMode;
	gint status;
} *pIRMCTL, IRMCTL;

pIRMCTL irmctl_loadconf(char *path);
void irmctl_activate(pACTION a);


#define getarg1(f) f->a1type?(gpointer)(&f->arg1.intarg):(gpointer)(f->arg1.chararg)
#define getarg2(f) f->a2type?(gpointer)(&f->arg2.intarg):(gpointer)(f->arg2.chararg)
#define irmctl_error_memory(message) { g_warning("%s: not enough memory\n", message); exit(EXIT_FAILURE); }

#endif
